@@ -24,7 +24,6 @@
 #include "nsGIFDecoder2.h"
 #include "nsIInputStream.h"
 #include "nsIComponentManager.h"
-#include "nsIImage.h"
 #include "nsMemory.h"
 
 #include "imgIContainerObserver.h"
@@ -33,52 +32,6 @@
 
 #include "nsRect.h"
 
-//////////////////////////////////////////////////////////////////////
-// Declaration of the static callbacks
-
-static int PR_CALLBACK BeginGIF(
-  void*    aClientData,
-  PRUint32 aLogicalScreenWidth, 
-  PRUint32 aLogicalScreenHeight,
-  PRUint8  aBackgroundRGBIndex);
-  
-static int PR_CALLBACK HaveDecodedRow(
-  void* aClientData,
-  PRUint8* aRowBufPtr,   // Pointer to single scanline temporary buffer
-  int aXOffset,          // With respect to GIF logical screen origin
-  int aLength,           // Length of the row?
-  int aRow,              // Row number?
-  int aDuplicateCount,   // Number of times to duplicate the row?
-  PRUint8 aDrawMode,     // il_draw_mode
-  int aInterlacePass);
-    
-static int PR_CALLBACK NewPixmap();
-
-static int PR_CALLBACK EndGIF(
-  void*    aClientData,
-  int      aAnimationLoopCount);
-  
-static int PR_CALLBACK BeginImageFrame(
-  void*    aClientData,
-  PRUint32 aFrameNumber,   /* Frame number, 1-n */
-  PRUint32 aFrameXOffset,  /* X offset in logical screen */
-  PRUint32 aFrameYOffset,  /* Y offset in logical screen */
-  PRUint32 aFrameWidth,    
-  PRUint32 aFrameHeight,   
-  GIF_RGB* aTransparencyChromaKey);
-static int PR_CALLBACK EndImageFrame(
-  void*    aClientData, 
-  PRUint32 aFrameNumber,
-  PRUint32 aDelayTimeout,
-  PRUint32 aDisposal);
-static int PR_CALLBACK SetupColorspaceConverter();
-static int PR_CALLBACK ResetPalette();
-static int PR_CALLBACK InitTransparentPixel();
-static int PR_CALLBACK DestroyTransparentPixel();
-
-static int PR_CALLBACK HaveImageAll(
-  void* aClientData);
-
 //////////////////////////////////////////////////////////////////////
 // GIF Decoder Implementation
 // This is an adaptor between GIF2 and imgIDecoder
@@ -86,21 +39,16 @@ static int PR_CALLBACK HaveImageAll(
 NS_IMPL_ISUPPORTS1(nsGIFDecoder2, imgIDecoder);
 
 nsGIFDecoder2::nsGIFDecoder2()
+  : mCurrentRow(-1)
+  , mLastFlushedRow(-1)
+  , mGIFStruct(nsnull)
+  , mAlphaLine(nsnull)
+  , mRGBLine(nsnull)
+  , mBackgroundRGBIndex(0)
+  , mCurrentPass(0)
+  , mLastFlushedPass(0)
+  , mGIFOpen(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
-  mImageFrame = nsnull;
-  
-  mGIFStruct = nsnull;
-
-  mAlphaLine = nsnull;
-  mRGBLine = nsnull;
-  mBackgroundRGBIndex = 0;
-
-  mCurrentRow = -1;
-  mLastFlushedRow = -1;
-
-  mCurrentPass = 0;
-  mLastFlushedPass = 0;
 }
 
 nsGIFDecoder2::~nsGIFDecoder2(void)
@@ -134,24 +82,12 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
   /* Always decode to 24 bit pixdepth */
   
   PRBool created = gif_create(&mGIFStruct);
-
   NS_ASSERTION(created, "gif_create failed");
+  if (!created)
+    return NS_ERROR_FAILURE;
 
   // Call GIF decoder init routine
-  GIFInit(
-    mGIFStruct,
-    this,
-    NewPixmap,
-    BeginGIF,
-    EndGIF,
-    BeginImageFrame,
-    EndImageFrame,
-    SetupColorspaceConverter,
-    ResetPalette,
-    InitTransparentPixel,
-    DestroyTransparentPixel,
-    HaveDecodedRow,
-    HaveImageAll);
+  GIFInit(mGIFStruct, this);
 
   return NS_OK;
 }
@@ -168,6 +104,12 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
 NS_IMETHODIMP nsGIFDecoder2::Close()
 {
   if (mGIFStruct) {
+    nsGIFDecoder2 *decoder = NS_STATIC_CAST(nsGIFDecoder2*, mGIFStruct->clientptr);
+    if (decoder->mImageFrame)
+      EndImageFrame(mGIFStruct->clientptr, mGIFStruct->images_decoded + 1, 
+                    mGIFStruct->delay_time);
+    if (decoder->mGIFOpen)
+      EndGIF(mGIFStruct->clientptr, mGIFStruct->loop_count);
     gif_destroy(mGIFStruct);
     mGIFStruct = nsnull;
   }
@@ -207,31 +149,34 @@ static NS_METHOD ReadDataOut(nsIInputStream* in,
 NS_METHOD
 nsGIFDecoder2::FlushImageData()
 {
-  PRInt32 width;
-  PRInt32 height;
-  mImageFrame->GetWidth(&width);
-  mImageFrame->GetHeight(&height);
+  PRInt32 imgWidth;
+  mImageContainer->GetWidth(&imgWidth);
+  nsRect frameRect;
+  mImageFrame->GetRect(frameRect);
+  
   switch (mCurrentPass - mLastFlushedPass) {
     case 0: {  // same pass
       PRInt32 remainingRows = mCurrentRow - mLastFlushedRow;
       if (remainingRows) {
-        nsRect r(0, mLastFlushedRow+1, width, remainingRows);
-        mObserver->OnDataAvailable(nsnull, nsnull, mImageFrame, &r);
+        nsRect r(0, frameRect.y + mLastFlushedRow + 1,
+                 imgWidth, remainingRows);
+        mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
       }    
     }
     break;
   
     case 1: {  // one pass on - need to handle bottom & top rects
-      nsRect r(0, 0, width, mCurrentRow+1);
-      mObserver->OnDataAvailable(nsnull, nsnull, mImageFrame, &r);
-      nsRect r2(0, mLastFlushedRow+1, width, height-mLastFlushedRow-1);
-      mObserver->OnDataAvailable(nsnull, nsnull, mImageFrame, &r2);
+      nsRect r(0, frameRect.y, imgWidth, mCurrentRow + 1);
+      mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
+      nsRect r2(0, frameRect.y + mLastFlushedRow + 1,
+                imgWidth, frameRect.height - mLastFlushedRow - 1);
+      mObserver->OnDataAvailable(nsnull, mImageFrame, &r2);
     }
     break;
 
     default: {  // more than one pass on - push the whole frame
-      nsRect r(0, 0, width, height);
-      mObserver->OnDataAvailable(nsnull, nsnull, mImageFrame, &r);
+      nsRect r(0, frameRect.y, imgWidth, frameRect.height);
+      mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
     }
   }
 
@@ -276,7 +221,7 @@ NS_IMETHODIMP nsGIFDecoder2::WriteFrom(nsIInputStream *inStr, PRUint32 count, PR
 //******************************************************************************
 
 //******************************************************************************
-int BeginGIF(
+int nsGIFDecoder2::BeginGIF(
   void*    aClientData,
   PRUint32 aLogicalScreenWidth, 
   PRUint32 aLogicalScreenHeight,
@@ -293,41 +238,43 @@ int BeginGIF(
   decoder->mBackgroundRGBIndex = aBackgroundRGBIndex;
 
   if (decoder->mObserver)
-    decoder->mObserver->OnStartDecode(nsnull, nsnull);
+    decoder->mObserver->OnStartDecode(nsnull);
 
   decoder->mImageContainer->Init(aLogicalScreenWidth, aLogicalScreenHeight, decoder->mObserver);
 
   if (decoder->mObserver)
-    decoder->mObserver->OnStartContainer(nsnull, nsnull, decoder->mImageContainer);
+    decoder->mObserver->OnStartContainer(nsnull, decoder->mImageContainer);
 
+  decoder->mGIFOpen = PR_TRUE;
   return 0;
 }
 
 //******************************************************************************
-int EndGIF(
+int nsGIFDecoder2::EndGIF(
     void*    aClientData,
     int      aAnimationLoopCount)
 {
   nsGIFDecoder2 *decoder = NS_STATIC_CAST(nsGIFDecoder2*, aClientData);
   if (decoder->mObserver) {
-    decoder->mObserver->OnStopContainer(nsnull, nsnull, decoder->mImageContainer);
-    decoder->mObserver->OnStopDecode(nsnull, nsnull, NS_OK, nsnull);
+    decoder->mObserver->OnStopContainer(nsnull, decoder->mImageContainer);
+    decoder->mObserver->OnStopDecode(nsnull, NS_OK, nsnull);
   }
   
   decoder->mImageContainer->SetLoopCount(aAnimationLoopCount);
   decoder->mImageContainer->DecodingComplete();
+
+  decoder->mGIFOpen = PR_FALSE;
   return 0;
 }
 
 //******************************************************************************
-int BeginImageFrame(
+int nsGIFDecoder2::BeginImageFrame(
   void*    aClientData,
   PRUint32 aFrameNumber,   /* Frame number, 1-n */
   PRUint32 aFrameXOffset,  /* X offset in logical screen */
   PRUint32 aFrameYOffset,  /* Y offset in logical screen */
   PRUint32 aFrameWidth,    
-  PRUint32 aFrameHeight,   
-  GIF_RGB* aTransparencyChromaKey) /* don't have this info yet */
+  PRUint32 aFrameHeight)
 {
   nsGIFDecoder2* decoder = NS_STATIC_CAST(nsGIFDecoder2*, aClientData);
   
@@ -337,15 +284,26 @@ int BeginImageFrame(
   decoder->mGIFStruct->width = aFrameWidth;
   decoder->mGIFStruct->height = aFrameHeight;
 
+  if (aFrameNumber == 1) {
+    // Send a onetime OnDataAvailable (Display Refresh) for the first frame
+    // if it has a y-axis offset.  Otherwise, the area may never be refreshed
+    // and the placeholder will remain on the screen. (Bug 37589)
+    PRInt32 imgWidth;
+    decoder->mImageContainer->GetWidth(&imgWidth);
+    if (aFrameYOffset > 0) {
+      nsRect r(0, 0, imgWidth, aFrameYOffset);
+      decoder->mObserver->OnDataAvailable(nsnull, decoder->mImageFrame, &r);
+    }
+  }
+
   return 0;
 }
 
 //******************************************************************************
-int EndImageFrame(
+int nsGIFDecoder2::EndImageFrame(
   void*    aClientData, 
   PRUint32 aFrameNumber,
-  PRUint32 aDelayTimeout,
-  PRUint32 aDisposal)  /* Time this frame should be displayed before the next frame 
+  PRUint32 aDelayTimeout)  /* Time this frame should be displayed before the next frame 
                               we can't have this in the image frame init because it doesn't
                               show up in the GIF frame header, it shows up in a sub control
                               block.*/
@@ -355,24 +313,41 @@ int EndImageFrame(
   // If mImageFrame hasn't been initialized, call HaveDecodedRow to init it
   // One reason why it may not be initialized is because the frame
   // is out of the bounds of the image.
-  if (!decoder->mImageFrame)
-    HaveDecodedRow(aClientData,nsnull,0,0,0,0,0,0);
-
-  // We actually have the timeout information before we get the lzw encoded image
-  // data, at least according to the spec, but we delay in setting the timeout for
-  // the image until here to help ensure that we have the whole image frame decoded before
-  // we go off and try to display another frame.
+  if (!decoder->mImageFrame) {
+    HaveDecodedRow(aClientData,nsnull,0,0,0);
+  } else {
+    // We actually have the timeout information before we get the lzw encoded 
+    // image data, at least according to the spec, but we delay in setting the 
+    // timeout for the image until here to help ensure that we have the whole 
+    // image frame decoded before we go off and try to display another frame.
+    decoder->mImageFrame->SetTimeout(aDelayTimeout);
+  }
   decoder->mImageContainer->EndFrameDecode(aFrameNumber, aDelayTimeout);
     
   if (decoder->mObserver && decoder->mImageFrame) {
-    decoder->mImageFrame->SetFrameDisposalMethod(aDisposal);
-
     decoder->FlushImageData();
 
+    if (aFrameNumber == 1) {
+      // If the first frame is smaller in height than the entire image, send a
+      // OnDataAvailable (Display Refresh) for the area it does not have data for.
+      // This will clear the remaining bits of the placeholder. (Bug 37589)
+      PRInt32 imgHeight;
+      PRInt32 realFrameHeight = decoder->mGIFStruct->height + decoder->mGIFStruct->y_offset;
+
+      decoder->mImageContainer->GetHeight(&imgHeight);
+      if (imgHeight > realFrameHeight) {
+        PRInt32 imgWidth;
+        decoder->mImageContainer->GetWidth(&imgWidth);
+
+        nsRect r(0, realFrameHeight, imgWidth, imgHeight - realFrameHeight);
+        decoder->mObserver->OnDataAvailable(nsnull, decoder->mImageFrame, &r);
+      }
+    }
+
     decoder->mCurrentRow = decoder->mLastFlushedRow = -1;
     decoder->mCurrentPass = decoder->mLastFlushedPass = 0;
 
-    decoder->mObserver->OnStopFrame(nsnull, nsnull, decoder->mImageFrame);
+    decoder->mObserver->OnStopFrame(nsnull, decoder->mImageFrame);
   }
 
   decoder->mImageFrame = nsnull;
@@ -381,26 +356,13 @@ int EndImageFrame(
   return 0;
 }
   
-
-
-//******************************************************************************
-// GIF decoder callback
-int HaveImageAll(
-  void* aClientData)
-{
-  return 0;
-}
-
 //******************************************************************************
 // GIF decoder callback notification that it has decoded a row
-int HaveDecodedRow(
+int nsGIFDecoder2::HaveDecodedRow(
   void* aClientData,
   PRUint8* aRowBufPtr,   // Pointer to single scanline temporary buffer
-  int aXOffset,          // With respect to GIF logical screen origin
-  int aLength,           // Length of the row?
   int aRowNumber,        // Row number?
   int aDuplicateCount,   // Number of times to duplicate the row?
-  PRUint8 aDrawMode,     // il_draw_mode
   int aInterlacePass)    // interlace pass (1-4)
 {
   nsGIFDecoder2* decoder = NS_STATIC_CAST(nsGIFDecoder2*, aClientData);
@@ -416,7 +378,7 @@ int HaveDecodedRow(
       format = gfxIFormats::RGB_A1;
     }
 
-#if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
     // XXX this works...
     format += 1; // RGB to BGR
 #endif
@@ -430,10 +392,11 @@ int HaveDecodedRow(
       return 0;
     }
 
+    decoder->mImageFrame->SetFrameDisposalMethod(decoder->mGIFStruct->disposal_method);
     decoder->mImageContainer->AppendFrame(decoder->mImageFrame);
 
     if (decoder->mObserver)
-      decoder->mObserver->OnStartFrame(nsnull, nsnull, decoder->mImageFrame);
+      decoder->mObserver->OnStartFrame(nsnull, decoder->mImageFrame);
 
     decoder->mImageFrame->GetImageBytesPerRow(&bpr);
     decoder->mImageFrame->GetAlphaBytesPerRow(&abpr);
@@ -458,16 +421,17 @@ int HaveDecodedRow(
 
     // XXX map the data into colors
     int cmapsize;
-    GIF_RGB* cmap;
+    PRUint8* cmap;
     cmapsize = decoder->mGIFStruct->global_colormap_size;
     cmap = decoder->mGIFStruct->global_colormap;
 
     if(decoder->mGIFStruct->global_colormap &&
        decoder->mGIFStruct->screen_bgcolor < cmapsize) {
       gfx_color bgColor = 0;
-      bgColor |= cmap[decoder->mGIFStruct->screen_bgcolor].red;
-      bgColor |= cmap[decoder->mGIFStruct->screen_bgcolor].green << 8;
-      bgColor |= cmap[decoder->mGIFStruct->screen_bgcolor].blue << 16;
+      PRUint32 bgIndex = decoder->mGIFStruct->screen_bgcolor * 3;
+      bgColor |= cmap[bgIndex];
+      bgColor |= cmap[bgIndex + 1] << 8;
+      bgColor |= cmap[bgIndex + 2] << 16;
       decoder->mImageFrame->SetBackgroundColor(bgColor);
     }
     if(decoder->mGIFStruct->local_colormap) {
@@ -475,105 +439,88 @@ int HaveDecodedRow(
       cmap = decoder->mGIFStruct->local_colormap;
     }
 
-    PRUint8* rgbRowIndex = decoder->mRGBLine;
-    PRUint8* rowBufIndex = aRowBufPtr;
-        
-    switch (format) {
-    case gfxIFormats::RGB:
-      {
-          if (cmap) {// cmap could have null value if the global color table flag is 0
-              while (rowBufIndex != decoder->mGIFStruct->rowend) {
-              #if defined(XP_MAC) || defined(XP_MACOSX)
-                  *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
-              #endif
-                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-                  ++rowBufIndex;
-            }
-          }
-          else 
-              memset(decoder->mRGBLine, 0, bpr);   
-          
-        for (int i=0; i<aDuplicateCount; i++)
-          decoder->mImageFrame->SetImageData(decoder->mRGBLine,
-                                             bpr, (aRowNumber+i)*bpr);
-      }
-      break;
-    case gfxIFormats::BGR:
-      {
-        if (cmap) {// cmap could have null value if the global color table flag is 0
-            while (rowBufIndex != decoder->mGIFStruct->rowend) {
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-                ++rowBufIndex;
-            }
-        }
-        else 
-            memset(decoder->mRGBLine, 0, bpr);   
-
-        for (int i=0; i<aDuplicateCount; i++)
-          decoder->mImageFrame->SetImageData(decoder->mRGBLine,
-                                             bpr, (aRowNumber+i)*bpr);
-      }
-      break;
-    case gfxIFormats::RGB_A1:
-    case gfxIFormats::BGR_A1:
-      {
-        gfx_color transColor = 0;
-        if (cmap && // cmap could have null value if the global color table flag is 0
-            decoder->mGIFStruct->is_transparent &&
-            (decoder->mGIFStruct->tpixel < cmapsize)) {
-          transColor |= cmap[decoder->mGIFStruct->tpixel].red;
-          transColor |= cmap[decoder->mGIFStruct->tpixel].green << 8;
-          transColor |= cmap[decoder->mGIFStruct->tpixel].blue << 16;
-        }
-        decoder->mImageFrame->SetTransparentColor(transColor);
-        memset(decoder->mRGBLine, 0, bpr);
-        memset(decoder->mAlphaLine, 0, abpr);
-        PRUint32 iwidth = (PRUint32)width;
-        if (cmap) {// cmap could have null value if the global color table flag is 0
-            for (PRUint32 x=0; x<iwidth; x++) {
-                if (*rowBufIndex != decoder->mGIFStruct->tpixel) {
-        #if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-        #else
-        #if defined(XP_MAC) || defined(XP_MACOSX)
-              *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
-        #endif
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-        #endif
-                decoder->mAlphaLine[x>>3] |= 1<<(7-x&0x7);
-              } else {
-        #if defined(XP_MAC) || defined(XP_MACOSX)
-                rgbRowIndex+=4;
-        #else
-                rgbRowIndex+=3;
-        #endif
-              }
-
-              ++rowBufIndex;
-            }
-        }
-        for (int i=0; i<aDuplicateCount; i++) {
-          decoder->mImageFrame->SetAlphaData(decoder->mAlphaLine,
+    if (!cmap) { // cmap could have null value if the global color table flag is 0
+      for (int i = 0; i < aDuplicateCount; ++i) {
+        if (format == gfxIFormats::RGB_A1 ||
+            format == gfxIFormats::BGR_A1) {
+          decoder->mImageFrame->SetAlphaData(nsnull,
                                              abpr, (aRowNumber+i)*abpr);
-          decoder->mImageFrame->SetImageData(decoder->mRGBLine,
-                                             bpr, (aRowNumber+i)*bpr);
+        }
+        decoder->mImageFrame->SetImageData(nsnull,
+                                           bpr, (aRowNumber+i)*bpr);
+      }
+    } else {
+      PRUint8* rgbRowIndex = decoder->mRGBLine;
+      PRUint8* rowBufIndex = aRowBufPtr;
+      
+      switch (format) {
+        case gfxIFormats::RGB:
+        case gfxIFormats::BGR:
+        {
+          while (rowBufIndex != decoder->mGIFStruct->rowend) {
+            PRUint32 colorIndex = (*rowBufIndex < cmapsize) ? (*rowBufIndex * 3) : 0;
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+            *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+            *rgbRowIndex++ = cmap[colorIndex + 1]; // green
+            *rgbRowIndex++ = cmap[colorIndex];     // red
+#else
+#if defined(XP_MAC) || defined(XP_MACOSX)
+            *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
+#endif
+            *rgbRowIndex++ = cmap[colorIndex];     // red
+            *rgbRowIndex++ = cmap[colorIndex + 1]; // green
+            *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+#endif
+            ++rowBufIndex;
+          }  
+          for (int i=0; i<aDuplicateCount; i++) {
+            decoder->mImageFrame->SetImageData(decoder->mRGBLine,
+                                               bpr, (aRowNumber+i)*bpr);
+          }
+          break;
+        }
+        case gfxIFormats::RGB_A1:
+        case gfxIFormats::BGR_A1:
+        {
+          memset(decoder->mRGBLine, 0, bpr);
+          memset(decoder->mAlphaLine, 0, abpr);
+          for (PRUint32 x = 0; x < (PRUint32)width; ++x) {
+            if (*rowBufIndex != decoder->mGIFStruct->tpixel) {
+              PRUint32 colorIndex = (*rowBufIndex < cmapsize) ? (*rowBufIndex * 3) : 0;
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+              *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+              *rgbRowIndex++ = cmap[colorIndex + 1]; // green
+              *rgbRowIndex++ = cmap[colorIndex];     // red
+#else
+#if defined(XP_MAC) || defined(XP_MACOSX)
+              *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
+#endif
+              *rgbRowIndex++ = cmap[colorIndex];     // red
+              *rgbRowIndex++ = cmap[colorIndex + 1]; // green
+              *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+#endif
+              decoder->mAlphaLine[x>>3] |= 1<<(7-x&0x7);
+            } else {
+#if defined(XP_MAC) || defined(XP_MACOSX)
+              rgbRowIndex+=4;
+#else
+              rgbRowIndex+=3;
+#endif
+            }
+            ++rowBufIndex;
+          }
+          for (int i=0; i<aDuplicateCount; i++) {
+            decoder->mImageFrame->SetAlphaData(decoder->mAlphaLine,
+                                               abpr, (aRowNumber+i)*abpr);
+            decoder->mImageFrame->SetImageData(decoder->mRGBLine,
+                                               bpr, (aRowNumber+i)*bpr);
+          }
+          break;
         }
       }
-      break;
-    default:
-      break;
-
     }
 
-    decoder->mCurrentRow = aRowNumber+aDuplicateCount-1;
+    decoder->mCurrentRow = aRowNumber + aDuplicateCount - 1;
     decoder->mCurrentPass = aInterlacePass;
     if (aInterlacePass == 1)
       decoder->mLastFlushedPass = aInterlacePass;   // interlaced starts at 1
@@ -581,39 +528,3 @@ int HaveDecodedRow(
 
   return 0;
 }
-
-//******************************************************************************
-int ResetPalette()
-{
-  return 0;
-}
-
-//******************************************************************************
-int SetupColorspaceConverter()
-{
-  return 0;
-}
-
-//******************************************************************************
-int EndImageFrame()
-{
-  return 0;
-}
-
-//******************************************************************************
-int NewPixmap()
-{
-  return 0;
-}
-
-//******************************************************************************
-int InitTransparentPixel()
-{
-  return 0;
-}
-
-//******************************************************************************
-int DestroyTransparentPixel()
-{
-  return 0;
-}
