@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: ckhelper.c,v $ $Revision: 1.26 $ $Date: 2002/08/30 22:56:57 $ $Name: MOZILLA_1_2_1_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: ckhelper.c,v $ $Revision: 1.28 $ $Date: 2003/01/30 03:02:55 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -47,6 +47,8 @@ static const char CVS_ID[] = "@(#) $RCSfile: ckhelper.c,v $ $Revision: 1.26 $ $D
 #include "ckhelper.h"
 #endif /* CKHELPER_H */
 
+extern const NSSError NSS_ERROR_DEVICE_ERROR;
+
 static const CK_BBOOL s_true = CK_TRUE;
 NSS_IMPLEMENT_DATA const NSSItem
 g_ck_true = { (CK_VOID_PTR)&s_true, sizeof(s_true) };
@@ -124,7 +126,7 @@ nssCKObject_GetAttributes
 	    ckrv != CKR_ATTRIBUTE_SENSITIVE) 
 	{
 	    nssSession_ExitMonitor(session);
-	    /* set an error here */
+	    nss_SetError(NSS_ERROR_DEVICE_ERROR);
 	    goto loser;
 	}
 	/* Allocate memory for each attribute. */
@@ -154,7 +156,7 @@ nssCKObject_GetAttributes
         ckrv != CKR_ATTRIBUTE_TYPE_INVALID &&
         ckrv != CKR_ATTRIBUTE_SENSITIVE) 
     {
-	/* set an error here */
+	nss_SetError(NSS_ERROR_DEVICE_ERROR);
 	goto loser;
     }
     if (alloced && arenaOpt) {
@@ -317,8 +319,7 @@ nssCryptokiCertificate_GetAttributes
   NSSDER *encodingOpt,
   NSSDER *issuerOpt,
   NSSDER *serialOpt,
-  NSSDER *subjectOpt,
-  NSSASCII7 **emailOpt
+  NSSDER *subjectOpt
 )
 {
     PRStatus status;
@@ -327,7 +328,7 @@ nssCryptokiCertificate_GetAttributes
     NSSSlot *slot;
     CK_ULONG template_size;
     CK_ATTRIBUTE_PTR attr;
-    CK_ATTRIBUTE cert_template[7];
+    CK_ATTRIBUTE cert_template[6];
     /* Set up a template of all options chosen by caller */
     NSS_CK_TEMPLATE_START(cert_template, attr, template_size);
     if (certTypeOpt) {
@@ -348,9 +349,6 @@ nssCryptokiCertificate_GetAttributes
     if (subjectOpt) {
 	NSS_CK_SET_ATTRIBUTE_NULL(attr, CKA_SUBJECT);
     }
-    if (emailOpt) {
-	NSS_CK_SET_ATTRIBUTE_NULL(attr, CKA_NETSCAPE_EMAIL);
-    }
     NSS_CK_TEMPLATE_FINISH(cert_template, attr, template_size);
     if (template_size == 0) {
 	/* caller didn't want anything */
@@ -395,9 +393,6 @@ nssCryptokiCertificate_GetAttributes
     if (subjectOpt) {
 	NSS_CK_ATTRIBUTE_TO_ITEM(&cert_template[i], subjectOpt); i++;
     }
-    if (emailOpt) {
-	NSS_CK_ATTRIBUTE_TO_UTF8(&cert_template[i], *emailOpt); i++;
-    }
     return PR_SUCCESS;
 }
 
